@@ -20,8 +20,10 @@
 #' Bayesian inference for linear modeling with regularizing priors on the 
 #' model parameters that are driven by prior beliefs about \eqn{R^2}, the 
 #' proportion of variance in the outcome attributable to the predictors. See 
-#' \code{\link{priors}} for an explanation of this critical point.
-#' 
+#' \code{\link{priors}} for an explanation of this critical point. 
+#' \code{\link{stan_glm}} with \code{family="gaussian"} also estimates a 
+#' linear model with normally-distributed errors and allows for various other 
+#' priors on the coefficients.
 #' 
 #' @export
 #' @templateVar fun stan_lm, stan_aov
@@ -111,6 +113,7 @@ stan_lm <- function(formula, data, subset, weights, na.action,
                     algorithm = c("sampling", "meanfield", "fullrank"), 
                     adapt_delta = NULL) {
   
+  algorithm <- match.arg(algorithm)
   call <- match.call(expand.dots = TRUE)
   mf <- match.call(expand.dots = FALSE)
   mf[[1L]] <- as.name("lm")
@@ -118,28 +121,31 @@ stan_lm <- function(formula, data, subset, weights, na.action,
   mf$qr <- FALSE
   mf$prior <- mf$prior_intercept <- mf$prior_PD <- mf$algorithm <- 
     mf$adapt_delta <- NULL
+  mf$method <- "model.frame"
   modelframe <- suppressWarnings(eval(mf, parent.frame()))
-  mt <- modelframe$terms
-  Y <- modelframe$y
-  X <- modelframe$x
-  if (!singular.ok) X <- X[, !is.na(modelframe$coefficients), drop = FALSE]
-  w <- modelframe$weights
-  offset <- model.offset(mf)
-  stanfit <- stan_lm.wfit(y = Y, x = X, w, offset, singular.ok = TRUE,
+  mt <- attr(modelframe, "terms")
+  Y <- model.response(modelframe, "numeric")
+  X <- model.matrix(mt, modelframe, contrasts)
+  w <- as.vector(model.weights(modelframe))
+  offset <- as.vector(model.offset(modelframe))
+  stanfit <- stan_lm.wfit(y = Y, x = X, w, offset, singular.ok = singular.ok,
                           prior = prior, prior_intercept = prior_intercept, 
                           prior_PD = prior_PD, 
                           algorithm = algorithm, adapt_delta = adapt_delta, 
                           ...)
-  fit <- nlist(stanfit, family = gaussian(), formula, offset, 
-               weights = w, x = X, y = Y, 
-               data = if(missing("data")) environment(formula) else data,
-               prior.info = prior, algorithm = "sampling",
-               call = call, terms = mt,
-               model = if (model) model.frame(modelframe) else NULL,
+  fit <- nlist(stanfit, family = gaussian(), formula, offset, weights = w,
+               x = X[,intersect(colnames(X), dimnames(stanfit)[[3]]), drop = FALSE], 
+               y = Y, data = if (missing("data")) environment(formula) else data,
+               prior.info = prior, 
+               algorithm, call, terms = mt,
+               model = if (model) modelframe else NULL,
                na.action = attr(modelframe, "na.action"),
                contrasts = attr(X, "contrasts"))
-  fit <- stanreg(fit)
-  if (!x) fit$x <- NULL
-  if (!y) fit$y <- NULL
-  fit
+  out <- stanreg(fit)
+  if (!x) 
+    out$x <- NULL
+  if (!y) 
+    out$y <- NULL
+  
+  return(out)
 }
