@@ -1,5 +1,5 @@
 # Part of the rstanarm package for estimating model parameters
-# Copyright (C) 2015 Trustees of Columbia University
+# Copyright (C) 2015, 2016 Trustees of Columbia University
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,19 +22,12 @@
 #' pages.
 #' 
 #' @name stanreg-methods
-#' @aliases VarCorr fixef ranef ngrps
+#' @aliases VarCorr fixef ranef ngrps sigma
 #' 
 #' @templateVar stanregArg object,x
 #' @template args-stanreg-object
 #' @param ... Ignored, except by the \code{update} method. See
 #'   \code{\link{update}}.
-#' @param parm For \code{confint}, an optional character vector of parameter
-#'   names.
-#' @param level For \code{confint}, a scalar between \eqn{0} and \eqn{1}
-#'   indicating the confidence level to use.
-#' @param correlation For \code{vcov}, if \code{FALSE} (the default) the
-#'   covariance matrix is returned. If \code{TRUE}, the correlation matrix is
-#'   returned instead.
 #' 
 #' @details Most of these methods are similar to the methods defined for objects
 #'   of class 'lm', 'glm', 'glmer', etc. However there are a few exceptions:
@@ -75,12 +68,6 @@
 #' }
 #' }
 #' 
-#' @note Because \code{sigma} is not yet included in \pkg{stats}, both 
-#'   \pkg{rstanarm} and \pkg{lme4} export a \code{sigma} generic. If both
-#'   packages are loaded it may be necessary to use \code{rstanarm::sigma} or
-#'   \code{lme4::sigma} (depending on which package is loaded first) in order to
-#'   access the appropriate method.
-#' 
 #' @seealso
 #' Other S3 methods for stanreg objects, which have separate documentation, 
 #' including \code{\link{as.matrix.stanreg}}, \code{\link{plot.stanreg}}, 
@@ -96,12 +83,19 @@ NULL
 #' @rdname stanreg-methods
 #' @export
 coef.stanreg <- function(object, ...) {
-  if (is.mer(object)) .mermod_coef(object, ...)
-  else object$coefficients
+  if (is.mer(object)) 
+    return(coef_mer(object, ...))
+  
+  object$coefficients
 }
 
 #' @rdname stanreg-methods
 #' @export
+#' @param parm For \code{confint}, an optional character vector of parameter
+#'   names.
+#' @param level For \code{confint}, a scalar between \eqn{0} and \eqn{1}
+#'   indicating the confidence level to use.
+#'
 confint.stanreg <- function(object, parm, level = 0.95, ...) {
   if (!used.optimizing(object)) {
     stop("For models fit using MCMC or a variational approximation please use ", 
@@ -122,20 +116,26 @@ fitted.stanreg <- function(object, ...)  {
 #' @export
 #' @keywords internal
 #' @param object Fitted model object.
-#' @param ... Arguments to methods.
+#' @param ... Arguments to methods. For example the
+#'   \code{\link[=stanreg-methods]{stanreg}} method accepts the argument
+#'   \code{newdata}.
 #' @return Pointwise log-likelihood matrix.
 #' @seealso \code{\link{log_lik.stanreg}}
+#' 
 log_lik <- function(object, ...) UseMethod("log_lik")
 
 #' @rdname stanreg-methods
 #' @export
-log_lik.stanreg <- function(object, ...) {
+#' @param newdata For \code{log_lik}, an optional data frame of new data (e.g.
+#'   holdout data). See \code{\link{posterior_predict}}.
+log_lik.stanreg <- function(object, newdata = NULL, ...) {
   if (!used.sampling(object)) 
     STOP_sampling_only("Pointwise log-likelihood matrix")
-  fun <- .llfun(object$family)
-  args <- .llargs(object)
+  fun <- ll_fun(family(object))
+  args <- ll_args(object, newdata)
   sapply(seq_len(args$N), function(i) {
-    as.vector(fun(i = i, data = args$data[i,, drop=FALSE], draws = args$draws)) 
+    as.vector(fun(i = i, data = args$data[i, , drop = FALSE], 
+                  draws = args$draws))
   })
 }
 
@@ -167,54 +167,67 @@ se <- function(object, ...) UseMethod("se")
 #' @rdname stanreg-methods
 #' @export
 se.stanreg <- function(object, ...) {
-  object$ses
+  ses <- object$ses
+  if (!is.mer(object))
+    return(ses)
+  unpad_reTrms(ses)
 }
 
 #' @rdname stanreg-methods
 #' @export
 #' @method update stanreg
 #' @param formula.,evaluate See \code{\link[stats]{update}}.
-#' 
+#'
 update.stanreg <- function(object, formula., ..., evaluate = TRUE) {
-  if (is.null(call <- getCall(object))) 
+  call <- getCall(object)
+  if (is.null(call)) 
     stop("'object' does not contain a 'call' component.", call. = FALSE)
   extras <- match.call(expand.dots = FALSE)$...
   if (!missing(formula.)) 
     call$formula <- update.formula(formula(object), formula.)
   if (length(extras)) {
     existing <- !is.na(match(names(extras), names(call)))
-    for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
+    for (a in names(extras)[existing]) 
+      call[[a]] <- extras[[a]]
     if (any(!existing)) {
       call <- c(as.list(call), extras[!existing])
       call <- as.call(call)
     }
   }
-  if (!evaluate) return(call)
-  else {
-    # do this like lme4 update.merMod instead of update.default
-    ff <- environment(formula(object))
-    pf <- parent.frame()
-    sf <- sys.frames()[[1L]]
-    tryCatch(eval(call, envir = ff),
-             error = function(e) {
-               tryCatch(eval(call, envir = sf),
-                        error = function(e) {
-                          eval(call, pf)
-                        })
-             })
-  }
+  
+  if (!evaluate) 
+    return(call)
+  
+  # do this like lme4 update.merMod instead of update.default
+  ff <- environment(formula(object))
+  pf <- parent.frame()
+  sf <- sys.frames()[[1L]]
+  tryCatch(eval(call, envir = ff),
+           error = function(e) {
+             tryCatch(eval(call, envir = sf),
+                      error = function(e) {
+                        eval(call, pf)
+                      })
+           })
 }
 
 #' @rdname stanreg-methods
 #' @export 
+#' @param correlation For \code{vcov}, if \code{FALSE} (the default) the
+#'   covariance matrix is returned. If \code{TRUE}, the correlation matrix is
+#'   returned instead.
+#'
 vcov.stanreg <- function(object, correlation = FALSE, ...) {
-  if (!is.mer(object)) out <- object$covmat
-  else {
+  if (!is.mer(object)) {
+    out <- object$covmat
+  } else {
     sel <- seq_along(fixef(object))
     out <- object$covmat[sel, sel, drop=FALSE]
   }
-  if (correlation) out <- cov2cor(out)
-  return(out)
+  if (!correlation) 
+    return(out)
+  
+  cov2cor(out)
 }
 
 
@@ -232,17 +245,17 @@ vcov.stanreg <- function(object, correlation = FALSE, ...) {
   as.list(object$glmod$reTrms$flist)
 }
 
-.mermod_coef <- function(object, ...) {
+coef_mer <- function(object, ...) {
   if (length(list(...))) 
     warning("Arguments named \"", paste(names(list(...)), collapse = ", "), 
             "\" ignored.", call. = FALSE)
   fef <- data.frame(rbind(fixef(object)), check.names = FALSE)
   ref <- ranef(object)
   refnames <- unlist(lapply(ref, colnames))
-  nmiss <- length(missnames <- setdiff(refnames, names(fef)))
+  missnames <- setdiff(refnames, names(fef))
+  nmiss <- length(missnames)
   if (nmiss > 0) {
-    fillvars <- setNames(data.frame(rbind(rep(0, nmiss))), 
-                         missnames)
+    fillvars <- setNames(data.frame(rbind(rep(0, nmiss))), missnames)
     fef <- cbind(fillvars, fef)
   }
   val <- lapply(ref, function(x) fef[rep.int(1L, nrow(x)), , drop = FALSE])
@@ -252,10 +265,10 @@ vcov.stanreg <- function(object, correlation = FALSE, ...) {
     nmsi <- colnames(refi)
     if (!all(nmsi %in% names(fef))) 
       stop("Unable to align random and fixed effects.", call. = FALSE)
-    for (nm in nmsi) val[[i]][[nm]] <- val[[i]][[nm]] + refi[, nm]
+    for (nm in nmsi) 
+      val[[i]][[nm]] <- val[[i]][[nm]] + refi[, nm]
   }
-  class(val) <- "coef.mer"
-  val
+  structure(val, class = "coef.mer")
 }
 
 #' @rdname stanreg-methods
@@ -265,7 +278,7 @@ vcov.stanreg <- function(object, correlation = FALSE, ...) {
 #' 
 fixef.stanreg <- function(object, ...) {
   coefs <- object$coefficients
-  coefs[.bnames(names(coefs), invert = TRUE)]
+  coefs[b_names(names(coefs), invert = TRUE)]
 }
 
 #' @rdname stanreg-methods
@@ -283,10 +296,10 @@ ngrps.stanreg <- function(object, ...) {
 #' @importFrom lme4 ranef
 #' 
 ranef.stanreg <- function(object, ...) {
-  if (used.optimizing(object)) 
-    sel <- .bnames(rownames(object$stan_summary))
-  else sel <- .bnames(object$stanfit@sim$fnames_oi)
-  ans <- object$stan_summary[sel, .select_median(object$algorithm)]
+  all_names <- if (used.optimizing(object))
+    rownames(object$stan_summary) else object$stanfit@sim$fnames_oi
+  sel <- b_names(all_names)
+  ans <- object$stan_summary[sel, select_median(object$algorithm)]
   # avoid returning the extra levels that were included
   ans <- ans[!grepl("_NEW_", names(ans), fixed = TRUE)]
   fl <- .flist(object)
@@ -306,36 +319,34 @@ ranef.stanreg <- function(object, ...) {
                check.names = FALSE)
   })
   names(ans) <- names(fl)
-  class(ans) <- "ranef.mer"
-  ans
+  structure(ans, class = "ranef.mer")
 }
 
-#' Extract residual standard deviation
-#' 
-#' @export
-#' @keywords internal
-#' @param object Fitted model object.
-#' @param ... Arguments to methods.
-sigma <- function(object, ...) UseMethod("sigma")
 
 #' @rdname stanreg-methods
 #' @export
-#' @method sigma stanreg
+#' @export sigma
+#' @rawNamespace if(getRversion()>='3.3.0') importFrom(stats, sigma) else
+#'   importFrom(lme4,sigma)
+#'
 sigma.stanreg <- function(object, ...) {
-  if (!("sigma" %in% rownames(object$stan_summary))) return(1)
-  else object$stan_summary["sigma", .select_median(object$algorithm)]
+  if (!("sigma" %in% rownames(object$stan_summary))) 
+    return(1)
+  
+  object$stan_summary["sigma", select_median(object$algorithm)]
 }
 
 #' @rdname stanreg-methods
-#' @param sigma,rdig Ignored (included for compatibility with
-#'   \code{\link[lme4]{VarCorr}}).
+#' @param sigma Ignored (included for compatibility with
+#'   \code{\link[nlme]{VarCorr}}).
 #' @export
 #' @export VarCorr
-#' @importFrom lme4 VarCorr mkVarCorr
-VarCorr.stanreg <- function(x, sigma = 1, rdig = 3) {
+#' @importFrom nlme VarCorr
+#' @importFrom lme4 mkVarCorr
+VarCorr.stanreg <- function(x, sigma = 1, ...) {
   cnms <- .cnms(x)
   means <- get_posterior_mean(x$stanfit)
-  means <- means[,ncol(means)]
+  means <- means[, ncol(means)]
   theta <- means[grepl("^theta_L", names(means))]
   sc <- sigma.stanreg(x)
   out <- lme4::mkVarCorr(sc = sc, cnms = cnms, 
@@ -347,6 +358,13 @@ VarCorr.stanreg <- function(x, sigma = 1, rdig = 3) {
 
 # Exported but doc kept internal ----------------------------------------------
 
+#' family method for stanreg objects
+#'
+#' @keywords internal
+#' @export
+#' @param object,... See \code{\link[stats]{family}}.
+family.stanreg <- function(object, ...) object$family
+
 #' model.frame method for stanreg objects
 #' 
 #' @keywords internal
@@ -364,7 +382,8 @@ model.frame.stanreg <- function(formula, fixed.only = FALSE, ...) {
     }
     return(fr)
   }
-  else NextMethod("model.frame")
+  
+  NextMethod("model.frame")
 }
 
 #' model.matrix method for stanreg objects
@@ -374,8 +393,10 @@ model.frame.stanreg <- function(formula, fixed.only = FALSE, ...) {
 #' @param object,... See \code{\link[stats]{model.matrix}}.
 #' 
 model.matrix.stanreg <- function(object, ...) {
-  if (is.mer(object)) object$glmod$X
-  else NextMethod("model.matrix")
+  if (is.mer(object))
+    return(object$glmod$X)
+  
+  NextMethod("model.matrix")
 }
 
 #' formula method for stanreg objects
@@ -387,14 +408,17 @@ model.matrix.stanreg <- function(object, ...) {
 #'   that both default to \code{FALSE}.
 #' 
 formula.stanreg <- function(x, ...) {
-  if (!is.mer(x)) return(x$formula)
-  else return(formula_mer(x, ...))
+  if (is.mer(x)) 
+    return(formula_mer(x, ...))
+  
+  x$formula
 }
 
 justRE <- function(f, response = FALSE) {
   response <- if (response && length(f) == 3) f[[2]] else NULL
   reformulate(paste0("(", vapply(lme4::findbars(f), 
-                                 function(x) paste(deparse(x, 500L), collapse = " "), 
+                                 function(x) paste(deparse(x, 500L), 
+                                                   collapse = " "), 
                                  ""), ")"), 
               response = response)
 }
@@ -414,39 +438,36 @@ formula_mer <- function (x, fixed.only = FALSE, random.only = FALSE, ...) {
     form <- attr(fr, "formula")
     form[[length(form)]] <- lme4::nobars(form[[length(form)]])
   }
-  if (random.only) {
+  if (random.only)
     form <- justRE(form, response = TRUE)
-  }
-  form
+
+  return(form)
 }
 
-#' terms method
-#' @export
-#' @keywords internal
-#' @param x,fixed.only,random.only,... See lme4:::terms.merMod
 #' terms method for stanreg objects
 #' @export
 #' @keywords internal
-#' @param x,fixed.only,random.only,... See lme4:::terms.merMod
+#' @param x,fixed.only,random.only,... See lme4:::terms.merMod.
 #' 
 terms.stanreg <- function(x, ..., fixed.only = TRUE, random.only = FALSE) {
-  if (!is.mer(x)) NextMethod("terms")
-  else {
-    fr <- x$glmod$fr
-    if (missing(fixed.only) && random.only) 
-      fixed.only <- FALSE
-    if (fixed.only && random.only) 
-      stop("'fixed.only' and 'random.only' can't both be TRUE.", call. = FALSE)
-
-    Terms <- attr(fr, "terms")
-    if (fixed.only) {
-      Terms <- terms.formula(formula(x, fixed.only = TRUE))
-      attr(Terms, "predvars") <- attr(terms(fr), "predvars.fixed")
-    } 
-    if (random.only) {
-      Terms <- terms.formula(lme4::subbars(formula.stanreg(x, random.only = TRUE)))
-      attr(Terms, "predvars") <- attr(terms(fr), "predvars.random")
-    }
-    return(Terms)
+  if (!is.mer(x))
+    return(NextMethod("terms"))
+  
+  fr <- x$glmod$fr
+  if (missing(fixed.only) && random.only) 
+    fixed.only <- FALSE
+  if (fixed.only && random.only) 
+    stop("'fixed.only' and 'random.only' can't both be TRUE.", call. = FALSE)
+  
+  Terms <- attr(fr, "terms")
+  if (fixed.only) {
+    Terms <- terms.formula(formula(x, fixed.only = TRUE))
+    attr(Terms, "predvars") <- attr(terms(fr), "predvars.fixed")
+  } 
+  if (random.only) {
+    Terms <- terms.formula(lme4::subbars(formula.stanreg(x, random.only = TRUE)))
+    attr(Terms, "predvars") <- attr(terms(fr), "predvars.random")
   }
+  
+  return(Terms)
 }
