@@ -22,31 +22,29 @@ SEED <- 12345
 set.seed(SEED)
 ITER <- 10
 CHAINS <- 2
-REFRESH <- ITER
+REFRESH <- 0
 
-stan_glm1 <- suppressWarnings(stan_glm(mpg ~ wt, data = mtcars, iter = ITER, 
-                                 chains = CHAINS,  seed = SEED, 
-                                 refresh = REFRESH))
-stan_glm_opt1 <- stan_glm(mpg ~ wt, data = mtcars, algorithm = "optimizing")
-stan_glm_vb1 <- update(stan_glm_opt1, algorithm = "meanfield", iter = 10000, 
-                       seed = SEED)
+SW <- suppressWarnings
+
+stan_glm1 <- SW(stan_glm(mpg ~ wt, data = mtcars, iter = ITER, 
+                         chains = CHAINS, seed = SEED, refresh = REFRESH))
+stan_glm_opt1 <- stan_glm(mpg ~ wt, data = mtcars, algorithm = "optimizing", 
+                          seed = SEED)
+stan_glm_vb1 <- update(stan_glm_opt1, algorithm = "meanfield", iter = 10000)
 glm1 <- glm(mpg ~ wt, data = mtcars)
 
 lmer1 <- lmer(diameter ~ (1|plate) + (1|sample), data = Penicillin)
-stan_lmer1 <- suppressWarnings(stan_lmer(diameter ~ (1|plate) + (1|sample), 
-                                        data = Penicillin, iter = ITER, 
-                                        chains = CHAINS, 
-                                        seed = SEED, refresh = REFRESH))
+stan_lmer1 <- SW(stan_lmer(diameter ~ (1|plate) + (1|sample), data = Penicillin, 
+                           iter = ITER, chains = CHAINS, seed = SEED, 
+                           refresh = REFRESH))
 lmer2 <- lmer(Reaction ~ Days + (Days | Subject), data = sleepstudy)
-stan_lmer2 <- suppressWarnings(stan_lmer(Reaction ~ Days + (Days | Subject), 
-                                        data = sleepstudy, iter = ITER, 
-                                        chains = CHAINS, 
-                                        seed = SEED, refresh = REFRESH))
+stan_lmer2 <- SW(stan_lmer(Reaction ~ Days + (Days | Subject), data = sleepstudy, 
+                           iter = ITER, chains = CHAINS, seed = SEED, 
+                           refresh = REFRESH))
 
-stan_polr1 <- suppressWarnings(stan_polr(tobgp ~ agegp, data = esoph,
-                                       prior = R2(0.2, "mean"), init_r = 0.1, 
-                                       iter = ITER, chains = CHAINS, 
-                                        seed = SEED, refresh = REFRESH))
+stan_polr1 <- SW(stan_polr(tobgp ~ agegp, data = esoph, prior = R2(0.2, "mean"), 
+                           init_r = 0.1, iter = ITER, chains = CHAINS, 
+                           seed = SEED, refresh = REFRESH))
 polr1 <- polr(tobgp ~ agegp, data = esoph, Hess = TRUE)
 
 att_names <- function(object) {
@@ -113,7 +111,7 @@ test_that("posterior_interval returns correct structure", {
   expect_silent(ci5 <- posterior_interval(stan_polr1, prob = 0.9))
   expect_identical(rownames(ci), c("(Intercept)", "wt", "sigma"))
   expect_identical(rownames(ci2), "wt")
-  expect_identical(rownames(ci3), rstanarm:::.bnames(rownames(example_model$stan_summary), value = TRUE)[1:15])
+  expect_identical(rownames(ci3), rstanarm:::b_names(rownames(example_model$stan_summary), value = TRUE)[1:15])
   expect_identical(rownames(ci4), c("(Intercept)", paste0("period", 2:4)))
   expect_identical(colnames(ci), c("25%", "75%"))
   expect_identical(colnames(ci2), c("2.5%", "97.5%"))
@@ -147,13 +145,21 @@ test_that("log_lik method works", {
   # and compare
   samp <- as.matrix(stan_glm1)
   y <- get_y(stan_glm1)
-  eta <- tcrossprod(get_x(stan_glm1), samp[, 1:2])
+  y_new <- y[1:10] + rnorm(10)
+  x <- get_x(stan_glm1)
+  x_new <- cbind(1, x[1:10, 2] + rnorm(10))
   sigma <- samp[, 3]
+  eta <- tcrossprod(x, samp[, 1:2])
+  eta_new <- tcrossprod(x_new, samp[, 1:2])
   llmat <- matrix(NA, nrow = nrow(samp), ncol = nrow(eta))
+  llmat_new <- matrix(NA, nrow = nrow(samp), ncol = nrow(eta_new))
   for (i in 1:nrow(llmat)) {
     llmat[i, ] <- dnorm(y, mean = eta[, i], sd = sigma[i], log = TRUE)
+    llmat_new[i, ] <- dnorm(y_new, mean = eta_new[, i], sd = sigma[i], log = TRUE)
   }
-  expect_equal(llmat, log_lik(stan_glm1))
+  expect_equal(log_lik(stan_glm1), llmat)
+  nd <- data.frame(mpg = y_new, wt = x_new[, 2])
+  expect_equal(log_lik(stan_glm1, newdata = nd), llmat_new)
 })
 
 test_that("ngrps is right", {
@@ -177,6 +183,24 @@ test_that("vcov returns correct structure", {
   expect_equal(dimnames(vcov(stan_lmer2)), dimnames(vcov(lmer2)))
 })
 
+test_that("sigma method works", {
+  # need to use :: because sigma is masked by lme4's sigma
+  rsigma <- rstanarm::sigma
+  expect_identical(rsigma(stan_polr1), 1)
+  expect_identical(rsigma(example_model), 1)
+  
+  expect_is(sig <- rsigma(stan_lmer1), "numeric")
+  expect_false(identical(sig, 1))
+  expect_is(sig <- rsigma(stan_lmer2), "numeric")
+  expect_false(identical(sig, 1))
+  expect_is(sig <- rsigma(stan_glm1), "numeric")
+  expect_false(identical(sig, 1))
+  expect_is(sig <- rsigma(stan_glm_vb1), "numeric")
+  expect_false(identical(sig, 1))
+  expect_is(sig <- rsigma(stan_glm_opt1), "numeric")
+  expect_false(identical(sig, 1))
+})
+
 test_that("VarCorr returns correct structure", {
   vc_lmer1 <- VarCorr(lmer1); vc_stan1 <- VarCorr(stan_lmer1)
   vc_lmer2 <- VarCorr(lmer2); vc_stan2 <- VarCorr(stan_lmer2)
@@ -252,7 +276,7 @@ test_that("as.matrix and as.data.frame methods work", {
   df <- as.data.frame(example_model)
   expect_identical(df, as.data.frame(mat))
   nc <- length(c(fixef(example_model), unlist(ranef(example_model))))
-  nr <- rstanarm:::.posterior_sample_size(example_model)
+  nr <- rstanarm:::posterior_sample_size(example_model)
   nms <- rownames(summary(example_model))[seq_len(nc)]
   expect_equal(dim(mat), c(nr, nc))
   expect_identical(colnames(mat), nms)
@@ -296,7 +320,7 @@ test_that("as.matrix and as.data.frame methods work", {
                  regexp = "'regex_pars' ignored")
 })
 
-test_that("print and summary methods don't throw errors", {
+test_that("print and summary methods ok for mcmc and vb", {
   expect_output(print(example_model, digits = 2), "stan_glmer")
   expect_output(print(example_model, digits = 2), "Error terms")
   expect_output(print(stan_lmer1, digits = 2), "stan_lmer")
@@ -325,22 +349,11 @@ test_that("print and summary methods don't throw errors", {
   expect_is(s, "summary.stanreg")
   expect_output(print(s), "stan_glmer")
   expect_output(print(s), paste("Posterior sample size:",  
-                                rstanarm:::.posterior_sample_size(example_model)))
+                                rstanarm:::posterior_sample_size(example_model)))
   expect_identical(attr(s, "algorithm"), "sampling")
   expect_identical(colnames(s), colnames(d))
   expect_identical(rownames(s), rownames(d))
   
-  expect_silent(s <- summary(stan_glm_opt1))
-  expect_silent(s <- summary(stan_glm_opt1, pars = c("wt", "sigma"), digits = 8))
-  expect_warning(s <- summary(stan_glm_opt1, regex_pars = c("wt", "sigma")), 
-               regexp = "'regex_pars' ignored")
-  expect_silent(d <- as.data.frame(s))
-  expect_is(s, "summary.stanreg")
-  expect_output(print(s), "stan_glm")
-  expect_identical(attr(s, "algorithm"), "optimizing")
-  expect_identical(colnames(s), colnames(d))
-  expect_identical(rownames(s), rownames(d))
-  
   expect_silent(s <- summary(stan_polr1, pars = "beta", probs = c(0.25, 0.75)))
   expect_silent(d <- as.data.frame(s))
   expect_identical(colnames(s), c("mean", "mcse", "sd", "25%", "75%", "n_eff", "Rhat"))
@@ -350,6 +363,8 @@ test_that("print and summary methods don't throw errors", {
   expect_is(s, "summary.stanreg")
   expect_output(print(s), "stan_polr")
   
+  expect_warning(s <- summary(stan_glm1, pars = "varying"), 
+                 regexp = "No group-specific parameters. 'varying' ignored.")
   expect_silent(s <- summary(stan_glm1, pars = c("alpha", "beta"), digits = 3))
   expect_is(s, "summary.stanreg")
   expect_output(print(s), "stan_glm")
@@ -362,6 +377,26 @@ test_that("print and summary methods don't throw errors", {
   expect_identical(attr(s, "algorithm"), "meanfield")
 })
 
+test_that("print and summary methods ok for optimization", {
+  expect_silent(s <- summary(stan_glm_opt1))
+  expect_silent(s <- summary(stan_glm_opt1, pars = c("wt", "sigma"), digits = 8))
+  expect_warning(s <- summary(stan_glm_opt1, regex_pars = c("wt", "sigma")), 
+                 regexp = "'regex_pars' ignored")
+  expect_silent(d <- as.data.frame(s))
+  expect_is(s, "summary.stanreg")
+  expect_output(print(s), "stan_glm")
+  expect_identical(attr(s, "algorithm"), "optimizing")
+  expect_identical(colnames(s), colnames(d))
+  expect_identical(rownames(s), rownames(d))
+  
+  counts <- c(18,17,15,20,10,20,25,13,12)
+  outcome <- gl(3,1,9)
+  treatment <- gl(3,3)
+  f <- counts ~ outcome + treatment
+  fit <- stan_glm.nb(f, algorithm = "optimizing", seed = SEED)
+  expect_output(print(fit), "overdispersion")
+})
+
 
 context("terms, formula, model.frame, and model.matrix methods")
 
@@ -422,3 +457,24 @@ test_that("formula works properly", {
   tmp$call <- NULL
   expect_error(formula(tmp), regexp = "can't find formula", ignore.case = TRUE)
 })
+
+test_that("update works properly", {
+  pss <- rstanarm:::posterior_sample_size
+  
+  fit <- SW(update(stan_lmer2, iter = ITER * 2, chains = 2 * CHAINS))
+  expect_equal(pss(fit), 4 * pss(stan_lmer2))
+  
+  fit <- SW(update(stan_glm1, iter = ITER * 2, chains = 2 * CHAINS))
+  expect_equal(pss(fit), 4 * pss(stan_glm1))
+  
+  call_only <- update(fit, evaluate = FALSE)
+  expect_is(call_only, "call")
+  expect_identical(call_only, getCall(fit))
+  
+  expect_warning(fit <- update(fit, algorithm = "optimizing"), 
+                 regexp = "unknown arguments: chains")
+  expect_identical(fit$algorithm, "optimizing")
+  
+  fit$call <- NULL
+  expect_error(update(fit), regexp = "does not contain a 'call' component")
+})
