@@ -20,10 +20,12 @@
 
 library(rstanarm)
 SEED <- 123
+set.seed(SEED)
 CHAINS <- 2
 ITER <- 100
-set.seed(SEED)
-REFRESH <- ITER
+REFRESH <- 0
+
+SW <- suppressWarnings
 
 plink <- function(fit, nd = NULL, sef = TRUE) 
   predict(fit, newdata = nd, type = "link", se.fit = sef)
@@ -39,9 +41,9 @@ test_that("predict ok for binomial", {
   SF <- cbind(numdead, numalive = 20-numdead)
   
   glmfit <- glm(SF ~ sex*ldose, family = binomial)
-  stanfit <- stan_glm(SF ~ sex*ldose, family = binomial, refresh = REFRESH,
-                      chains = CHAINS, iter = ITER, seed = SEED)
-  stanfit_opt <- update(stanfit, algorithm = "optimizing")
+  stanfit <- SW(stan_glm(SF ~ sex*ldose, family = binomial, chains = CHAINS, 
+                         iter = ITER, seed = SEED, refresh = REFRESH))
+  stanfit_opt <- SW(update(stanfit, algorithm = "optimizing"))
   
   
   pg <- plink(glmfit)
@@ -69,9 +71,9 @@ test_that("predict ok for binomial", {
 
 test_that("predict ok for gaussian", {
   glmfit <- glm(mpg ~ wt, data = mtcars)
-  stanfit <- stan_glm(mpg ~ wt, data = mtcars, refresh = 2 * REFRESH,
-                      chains = CHAINS, iter = 2 * ITER, seed = SEED)
-  stanfit_opt <- update(stanfit, algorithm = "optimizing")
+  stanfit <- SW(stan_glm(mpg ~ wt, data = mtcars, chains = CHAINS,
+                      iter = 2 * ITER, seed = SEED, refresh = REFRESH))
+  stanfit_opt <- SW(update(stanfit, algorithm = "optimizing"))
   
   pg <- plink(glmfit)
   ps <- plink(stanfit)
@@ -99,10 +101,9 @@ test_that("predict ok for Poisson", {
                     outcome = gl(3,1,9), treatment = gl(3,3))
 
   glmfit <- glm(counts ~ outcome + treatment, data = dat, family = poisson())
-  stanfit <- stan_glm(counts ~ outcome + treatment, 
-                      data = dat, family = poisson(), refresh = REFRESH,
-                      chains = CHAINS, iter = ITER, seed = SEED)
-  stanfit_opt <- update(stanfit, algorithm = "optimizing")
+  stanfit <- SW(stan_glm(counts ~ outcome + treatment, data = dat, family = poisson(), 
+                         chains = CHAINS, iter = ITER, seed = SEED, refresh = REFRESH))
+  stanfit_opt <- SW(update(stanfit, algorithm = "optimizing"))
 
   pg <- plink(glmfit)
   ps <- plink(stanfit)
@@ -127,4 +128,3 @@ test_that("predict ok for Poisson", {
   expect_equal(pg$se.fit, pso$se.fit, tol = 0.1)
   expect_equal(presp(glmfit, newd)[1:2], presp(stanfit_opt, newd), tol = 0.1)
 })
-
