@@ -46,10 +46,12 @@ test_that("stan_lm returns expected result for trees example", {
 
 test_that("stan_lm doesn't break with less common priors", {
   # prior = NULL
-  expect_output(fit <- stan_lm(mpg ~ ., data = mtcars, prior = NULL,
+  expect_output(fit <- stan_lm(mpg ~ -1 + ., data = mtcars, prior = NULL,
                 iter = 10, chains = 1, seed = SEED), regexp = "SAMPLING")
   # prior_intercept = normal()
-  expect_output(update(fit, prior_intercept = normal()), regexp = "SAMPLING")
+  expect_output(fit <- stan_lm(mpg ~ ., data = mtcars, 
+                               prior = R2(0.75), prior_intercept = normal(),
+                               iter = 10, chains = 1, seed = SEED), regexp = "SAMPLING")
 })
 
 test_that("stan_lm doesn't break with vb algorithms", {
@@ -61,6 +63,11 @@ test_that("stan_lm doesn't break with vb algorithms", {
                 regexp = "Automatic Differentiation Variational Inference")
 })
 
+test_that("stan_lm throws error if only intercept", {
+  expect_error(stan_lm(mpg ~ 1, data = mtcars, prior = R2(location = 0.75)), 
+               regexp = "not suitable for estimating a mean")
+})
+
 context("stan_aov")
 test_that("stan_aov returns expected result for npk example", {
   fit <- stan_aov(yield ~ block + N*P*K, data = npk, contrasts = "contr.poly",
@@ -71,5 +78,3 @@ test_that("stan_aov returns expected result for npk example", {
   expect_output(print(fit), regexp = "stan_aov")
   expect_output(print(fit), regexp = "ANOVA-like table")
 })
-
-
